 /***************************************************************
 *                                                              *  
 *    o_convert.h                                               *
 *    image/color conversion routines                           *
 *                                                              *
 *    (G) corTeX / Optimum   1999-02-04                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/

/*
 * The code is split into 2 source files:
 * o_convert1.S contains the assembly routines
 * o_convert2.c contains the C routines, slower but
 *              covering the lacks of the .S
 */

/*
 *  Note this is still very alpha,
 *  many conversion combination are not supported.
 */


#ifndef O_CONVERT_H
#define O_CONVERT_H

#include "o_types.h"


/*
  a struct for look-up tables :
  source and dest are described in term of bit per indice, and
  of effective bit size (like bpp vs depth)
  ( NOT USED YET)
*/

typedef struct{
  int source_bit;  /* 8, 15/16 */
  int source_align;/* 8, 16 */

  int dest_bit;    /* 8, 15,16, 24, 32 */
  int dest_align;  /* 8, 16, (24?), 32 */
  
  void *table;
} ot_lookup;


/* quelques routines de conversion... 
 * ne pas vous affoler en voyant toutes ces declaration:
 * Un meme codage couleur peut posseder 4 noms differents, selon
 * le point de vue ou on le decrit :
 * par exemple pour un mode RGB (depth 24bits, mais alignement 32 bits):
 *             lARGB pour un int [ARGB] sur une machine Less significant byte first
 *             mBGRA pour un int [BGRA]                 Most
 *             bBGRA pour 4 octets B-G-R-A
 *              ARGB pour un int [ARGB] sur la machine actuellement utilisee
 * => les 3 premieres denominations sont identiques, ainsi que la 4eme sur un PC
 * 
 * Donc en pratique, on utilisera plutot le nom sans prefixe "par defaut"
 * sauf pour specifier sans ambiguit un format(enfin c'est vous qui voyez,
 * et puis de toute facon tout sera bien explique dans l'optilib finale... ;) )
 * 
 * Evidemment ca se casse la gueule pour le 16bits, vu qu'une composante est
 * a cheval sur 2octets... donc ici RGB_565 = un short [RGB] vu par un PC
 *
 * Les autres declarations sont : 
 *  P  pour un 'pixel' general (dont la structure interne
 *   importe peu, on met donc seulement sa taille en bits )
 * Is used for palet mode in 8bits
 */

/* 32 bits -> 16 bits */
extern void ofn_cvrt_ARGB_8888_RGB_565 (uint32 *src, uint16 *dst, int nbpixels);
extern void ofn_cvrt_bBGRA_8888_RGB_565(uint32 *src, uint16 *dst, int nbpixels);
extern void ofn_cvrt_mBGRA_8888_RGB_565(uint32 *src, uint16 *dst, int nbpixels);
extern void ofn_cvrt_lARGB_8888_RGB_565(uint32 *src, uint16 *dst, int nbpixels);

/* 32 bits -> 16 bits */
void ofn_cvrt_RGBA_8888_RGB_565 (uint32 *src, uint16 *dst, int nbpixels);
#define ofn_cvrt_bABGR_8888_RGB_565 ofn_cvrt_RGBA_8888_RGB_565
#define ofn_cvrt_mABGR_8888_RGB_565 ofn_cvrt_RGBA_8888_RGB_565
#define ofn_cvrt_lRGBA_8888_RGB_565 ofn_cvrt_RGBA_8888_RGB_565


	
/* 16 bits -> 32 */
extern void ofn_cvrt_RGB_565_ARGB_8888 (uint16 *src, uint32 *dst, int nbpixels);
extern void ofn_cvrt_RGB_565_bBGRA_8888(uint16 *src, uint32 *dst, int nbpixels);
extern void ofn_cvrt_RGB_565_mBGRA_8888(uint16 *src, uint32 *dst, int nbpixels);
extern void ofn_cvrt_RGB_565_lARGB_8888(uint16 *src, uint32 *dst, int nbpixels);

/* 32 bits exotique ?  -> 16 */
extern void ofn_cvrt_ABGR_8888_RGB_565 (uint32 *src, uint16 *dst, int nbpixels);
extern void ofn_cvrt_bRGBA_8888_RGB_565(uint32 *src, uint16 *dst, int nbpixels);
extern void ofn_cvrt_mRGBA_8888_RGB_565(uint32 *src, uint16 *dst, int nbpixels);
extern void ofn_cvrt_lABGR_8888_RGB_565(uint32 *src, uint16 *dst, int nbpixels);

/* 16 bits -> 32 */
extern void ofn_cvrt_RGB_565_ABGR_8888 (uint16 *src, uint32 *dst, int nbpixels);
extern void ofn_cvrt_RGB_565_bRGBA_8888(uint16 *src, uint32 *dst, int nbpixels);
extern void ofn_cvrt_RGB_565_mRGBA_8888(uint16 *src, uint32 *dst, int nbpixels);
extern void ofn_cvrt_RGB_565_lABGR_8888(uint16 *src, uint32 *dst, int nbpixels);



/* 32 bits -> 15 bits */
void ofn_cvrt_ABGR_8888_RGB_555(uint32 *src, uint16 *dst, int nbpixels);

/* 16 bits -> 15 bits */
void ofn_cvrt_RGB_565_RGB_555(uint16 *src, uint16 *dst, int nbpixels);

/* palet 8 bits -> 16 bits  */
extern void ofn_cvrt_P_8_P_16(uint8 *src, uint16 *dst, int nbpixels, uint16 *palet);

/* palet 8 bits -> 32 bits  */
extern void ofn_cvrt_P_8_P_32(uint8 *src, uint32 *dst, int nbpixels, uint32 *palet);


/* palet 15/16 bits -> 8 bits (brute force conversion method) */
/* RAM consuming (?) but  permit special true optimized palets     */
void ofn_cvrt_P_16_P_8(uint16 *src, uint8 *dst, int nbpixels, uint8 *palet);

/* 16 bits -> 332 8 bits directcolor
 *  "computed" conversion (instead of palet);
 * can only handle 332 general 8bits palette, so can be quite ugly
 */
void ofn_cvrt_RGB_565_RGB_332(uint16 *src, uint8 *dst, int nbpixels);

/* 32 bits -> 332 8 bits directcolor  (same remark) */
/* [ hum ... P8 ? or RGB332 ?... ] */

void ofn_cvrt_ARGB_8888_RGB_332(uint32 *src, uint8 *dst, int nbpixels);
#define ofn_cvrt_lARGB_8888_RGB_332 ofn_cvrt_ARGB_8888_RGB_332
#define ofn_cvrt_bBGRA_8888_RGB_332 ofn_cvrt_ARGB_8888_RGB_332
#define ofn_cvrt_mBGRA_8888_RGB_332 ofn_cvrt_ARGB_8888_RGB_332

void ofn_cvrt_BGRA_8888_RGB_332(uint32 *src, uint8 *dst, int nbpixels);

/* 32 bits <-> 32 bits */
void ofn_cvrt_ARGB_8888_RGBA_8888(uint32 *src, uint32 *dst, int nbpixels);
void ofn_cvrt_RGBA_8888_ARGB_8888(uint32 *src, uint32 *dst, int nbpixels);

void ofn_cvrt_ARGB_8888_ABGR_8888(uint32 *src, uint32 *dst, int nbpixels);
void ofn_cvrt_ABGR_8888_ARGB_8888(uint32 *src, uint32 *dst, int nbpixels);


#endif

