 /***************************************************************
 *                                                              *  
 *    o_gfxm.h                                                  *
 *    #define's for graphic modes                               *
 *                                                              *
 *    (G) corTeX / Optimum   1999-02-03                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/


#ifndef O_GFXM_H
#define O_GFXM_H

#include <stdarg.h>   // for alternate parameters
#include "o_convert.h"

#define INTEL_86   // I will change this to "BigEndian / LittleEndian "... one day



#define MODE_UNKNOWN   (-1)
#define MODE_RGBA_8888 1
#define MODE_BGRA_8888 2

#define MODE_ARGB_8888 3
#define MODE_ABGR_8888 4


#define MODE_RGB_888 8
#define MODE_BGR_888 9
#define MODE_RGB_565 10
#define MODE_RGB_555 11
#define MODE_ARGB_1555 MODE_RGB_555
#define MODE_PAL_8  16


#define MODE_GRAY_8  17
#define MODE_RGB_332  18

#define MODE_MAX   18



// virtual mode, used only for Color component order
// (nb : numbers are chosen as follow :
//   bit 0 = 1 if G--R
//   bit 1 = 1 if B--R
//   bit 2 = 1 if B--G
// of course every combination does not make sense ...)

#define MODE_RGB  0
#define MODE_GRB  1

#define MODE_GBR  3
#define MODE_RBG  4

#define MODE_BRG  6
#define MODE_BGR  7

#define MODE_XXX (-1)





#ifdef SPARC

// still searching how to name the 'std' mode (those outside the
// arch-dependant parts)... keep then non-prefixed, or the byte-ordered?..

#define MODE_bRGBA_8888  MODE_RGBA_8888       /* in byte-order convention */
#define MODE_mRGBA_8888  MODE_RGBA_8888       /* msb Word */
#define MODE_lABGR_8888  MODE_RGBA_8888       /* lsb Word */

#define MODE_bBGRA_8888  MODE_BGRA_8888  
#define MODE_mBGRA_8888  MODE_BGRA_8888  
#define MODE_lARGB_8888  MODE_BGRA_8888  

#define MODE_bRGB_888    MODE_RGB_888
#endif


#ifdef INTEL_86

#define MODE_bRGBA_8888  MODE_ABGR_8888       /* in byte-order convention */
#define MODE_mRGBA_8888  MODE_ABGR_8888       /* msb Word */
#define MODE_lABGR_8888  MODE_ABGR_8888       /* lsb Word */

#define MODE_bBGRA_8888  MODE_ARGB_8888  
#define MODE_mBGRA_8888  MODE_ARGB_8888  
#define MODE_lARGB_8888  MODE_ARGB_8888  

#define MODE_bRGB_888    MODE_RGB_888
#endif


// conversion method to use

#define CONVERT_DEFAULT    0
#define CONVERT_COMPUTE    1
#define CONVERT_PALETTE    2

/*****************/
/* generic stuff */


// returns the opti_gfx mode (RGB_565, etc) according to the
// bpp and color mask passed.
// (return MODE_RGB_565 for example)
int ofn_GetGfxMode(int bpp, int depth, int rmask, int gmask, int bmask);

// return RGB component order according to masks
// (return MODE_RGB for example)
int ofn_GetColorOrder( int rmask, int gmask, int bmask);

// return the component order of the specified MODE_ 
int ofn_GetOrderOfMode( int gfxMode);

// return the bpp (for memory allocation..) of the mode
// *!*!* This is BITS per pixel, so you must /8 for the malloc !*!*!
int ofn_GetBppOfMode( int gfxMode);

// return the color depth (int BITS)
int ofn_GetDepthOfMode( int gfxMode);

// return textual name
char *ofn_GetNameOfMode(int gfxMode);

/**********************/
/* Conversion Wrapper */

// basic syntaxe only needs source and target buffers
// and gfxMode, and the number of pixels to convert.
// additionnal parameters are passed when conversion
// use extra routines (like palette or lookup table).
// 'special' clear ambiguities (for example 16->8bits
// can be used in computed 332 (no extra var), or using 
// a 'giant' lookup-table (one extra var).
// Semantic of 'special' and additional parameters
// are completely special-case dependant :)
void ofn_ConvertMode(void *source, void *dest,
		   int sMode, int dMode,
		   int nbpixels,
		   int special,
		   ...);


#endif



