 /***************************************************************
 *                                                              *  
 *    o_handler.h                                               *
 *    handler of (X)Events                                      *
 *                                                              *
 *    (G) corTeX / Optimum   1999-05-08                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/

#ifndef O_HANDLER_H
#define O_HANDLER_H

#include "o_types.h"
#include "o_std.h"

//#include "o_schedule.h"
#include "o_surface.h"
#include "o_xutils.h"


/**************************************************
 * Following things are not implemented or not used
 * or not usefull or ....
 **************************************************/


// constants to know wich type we are really recording in
// the ot_anything typecasted field .......

#define OPL_REG_NOTHING         (-1)

#define OPL_REG_OT_TIME         8

//#define OPL_REG_OT_PHYSURFACE   16
//#define OPL_REG_OT_SURFACE      17
#define OPL_REG_OT_SURFACE_LIST    18



/* "structures" to record everything and nothing, using void * ...
   (an objet-oriented optilib would be cleaner regarding to this)

   Everything that must be somewhat updated upon events/signals etc...
   are registered here, and the all-purpose handler examine the list
   to act on it.
*/
 
typedef struct ot_handler_item{
  int32                   element_type;   // one OPL_REG_xxx
  ot_anything            *element;
  struct ot_handler_item *next;
  struct ot_handler_item *previous;
} ot_handler_item;

typedef struct{
  int              lock;  // XXX  there should be a semaphore or something like this
  ot_handler_item *list;
} ot_handler_registry;



// init the field(s)
void ofn_HandlerRegistryInit(ot_handler_registry *oreg);

// free the chained list of the registry (but certainly not the
// elements registered, don't worry)
void ofn_HandlerRegistryDestroy(ot_handler_registry *oreg);

// generic function to add some void* element to the registry
void ofn_HandlerRegistryAddElement(ot_handler_registry *oreg,
				   int element_type,
				   ot_anything *element);

// generic function to remove some void* element from the registry
void ofn_HandlerRegistryRemoveElement(ot_handler_registry *oreg,
				      int element_type,  // perhaps not very useful ?
				      ot_anything *element);

// same thing with explicit types (high level wrappers ,)  )
// simple Macros

#define ofn_HandlerRegistryAdd_time(oreg, time_el)    ofn_HandlerRegistryAddElement(oreg, OPL_REG_OT_TIME, (ot_anything *)(time_el))
#define ofn_HandlerRegistryRemove_time(oreg, time_el) ofn_HandlerRegistryRemoveElement(oreg, OPL_REG_OT_TIME, (ot_anything *)(time_el))

#define ofn_HandlerRegistryAdd_surface_list(oreg, surface_list_el)    \
ofn_HandlerRegistryAddElement(oreg, OPL_REG_OT_SURFACE_LIST, (ot_anything *)(surface_list_el))
#define ofn_HandlerRegistryRemove_surface_list(oreg, surface_list_el) \
ofn_HandlerRegistryRemoveElement(oreg, OPL_REG_OT_SURFACE_LIST, (ot_anything *)(surface_list_el))


/**************************************************
 * Now these are some useful routines
 * 
 **************************************************/


// redefine Events, like X does....

#define OPL_EV_NOTHING  0
#define OPL_EV_KEY      1

typedef struct{
  int type;    // OPL_EV_KEY
  int pressed; // 'true' = pressed; 0=released 
               /* XXX or 0 : released 1 : pressed 2:autorepeat ? */
  int keysym;  // X keysym constants
} ot_ev_key;

typedef union{
  int type;    // OPL_EV_xxxx
  ot_ev_key ev_key;
} ot_event;


/* Process pending XEvents pending for the window associated with the
 * surface_list. 
 * Every events that can be processed internally (Window redraw,
 * shm completion,...) are removed from the queue and are used to do what must
 * be done.
 * (XXX not shm, done by o_xutils when blitting)
 * After that, the first remaining event is converted and returned to
 * the caller. This function must be called multiple times if several 
 * such events are present.
 */

/*
** Assume that all surfaces belong to the same Xwindow
*/

/*
** Hum at least it WILL do that when i code it ;)
**
** XXX : add a 'mask' parameter for the surfacelist ?...
*/
void ofn_ProcessSurfaceListEvents( ot_surface_list *osurlf, ot_event *oev);



#endif













