 /***************************************************************
 *                                                              *
 *    o_convert2.c                                              *
 *    image/color conversion routines                           *
 *    C routines                                                *
 *                                                              *
 *    (G) corTeX / Optimum   1999-02-04                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/


#include "o_convert.h"


void ofn_cvrt_ABGR_8888_RGB_555(uint32 *src, uint16 *dst, int nbpixels){
  int i;
  int r,g,b;
  uint8 *p;


  p = (uint8 *)src;

  for (i=0; i<nbpixels; i++){
    r = *(p+4*i );
    g = *(p+4*i +1);
    b = *(p+4*i +2);

    r >>=3;             // color reduction
    g >>=3;
    b >>=3;

    *(dst+i) = ((r<<10) | (g<<5) | (b));  // 555
    
  }
}


void ofn_cvrt_RGB_565_RGB_555(uint16 *src, uint16 *dst, int nbpixels){

  int i;
  uint16 p;

  for (i=0; i<nbpixels; i++){
    p = *(src + i);
    *(dst+i) = (p & 0x1f) | ((p & (~0xfc)) >>1);  // suppress one bit of green
  }
}


void ofn_cvrt_P_16_P_8(uint16 *src, uint8 *dst, int nbpixels, uint8 *palet){
   int i;
   for (i=0; i<nbpixels; i++)
     *(dst+i) = *(palet + *(src+i));
 }
void ofn_cvrt_P_8_P_32(uint8 *src, uint32 *dst, int nbpixels, uint32 *palet){
   int i;
   for (i=0; i<nbpixels; i++)
     *(dst+i) = *(palet + *(src+i));
 }

void ofn_cvrt_RGB_565_RGB_332(uint16 *src, uint8 *dst, int nbpixels){
  int i;
  int r,g,b;
  uint16 p;

  for (i=0; i<nbpixels; i++){
    p = *(src+i);

    r = (p>>11) & 0x1f ;    // extract color components
    g = (p>>5) & 0x3f;
    b = p & 0x1f ;

    r >>=2;             // color reduction
    g >>=3;
    b >>=3;

    *(dst+i) = ((r<<5) | (g<<2) | (b));  // 332 8bit colors
    
  }
}

//   o_cvrt_bBGRA_8888_P_8
void ofn_cvrt_ARGB_8888_RGB_332(uint32 *src, uint8 *dst, int nbpixels){
  int i;
  int r,g,b;
  uint8 *p;


  p = (uint8 *)src;

  for (i=0; i<nbpixels; i++){
    r = *(p+4*i +2);
    g = *(p+4*i +1);
    b = *(p+4*i );

    r >>=5;             // color reduction
    g >>=5;
    b >>=6;

    *(dst+i) = ((r<<5) | (g<<2) | (b));  // 332 8bit colors
    
  }
}

//   ofn_cvrt_bARGB_8888_P_8
void ofn_cvrt_BGRA_8888_RGB_332(uint32 *src, uint8 *dst, int nbpixels){
  int i;
  int r,g,b;
  uint8 *p;


  p = (uint8 *)src;

  for (i=0; i<nbpixels; i++){
    r = *(p+4*i +1);
    g = *(p+4*i +2);
    b = *(p+4*i +3);

    r >>=5;             // color reduction
    g >>=5;
    b >>=6;

    *(dst+i) = ((r<<5) | (g<<2) | (b));  // 332 8bit colors
    
  }
}




/* 32 bits -> 32 bits */
void ofn_cvrt_ARGB_8888_RGBA_8888(uint32 *src, uint32 *dst, int nbpixels){
  int i;
  for (i=0; i<nbpixels; i++)
    *(dst+i) = *(src+i) << 8;
}

void ofn_cvrt_RGBA_8888_ARGB_8888(uint32 *src, uint32 *dst, int nbpixels){
  int i;
  for (i=0; i<nbpixels; i++)
    *(dst+i) = *(src+i) >> 8;

}

void ofn_cvrt_ARGB_8888_ABGR_8888(uint32 *src, uint32 *dst, int nbpixels){
  int i;
  uint8 *s = (uint8 *)src;
  uint8 *d = (uint8 *)dst;
  for (i=0; i<nbpixels; i++){
    *(d+i*4+2) = *(s+i*4);         // b
    *(d+i*4+1) = *(s+i*4+1);       // g
    *(d+i*4)   = *(s+i*4+2);       // r
  }
}

void ofn_cvrt_ABGR_8888_ARGB_8888(uint32 *src, uint32 *dst, int nbpixels){
  int i;
  uint8 *s = (uint8 *)src;
  uint8 *d = (uint8 *)dst;
  for (i=0; i<nbpixels; i++){
    *(d+i*4)   = *(s+i*4+2);         // r
    *(d+i*4+1) = *(s+i*4+1);       // g
    *(d+i*4+2) = *(s+i*4);       // b
  }
}




void ofn_cvrt_RGBA_8888_RGB_565 (uint32 *src, uint16 *dst, int nbpixels){
  int i;
  int r,g,b;
  uint8 *p;


  p = (uint8 *)src;

  for (i=0; i<nbpixels; i++){
    r = *(p+4*i +3);
    g = *(p+4*i +2);
    b = *(p+4*i +1);

    r >>=3;       
    g >>=2;
    b >>=3;

    *(dst+i) = ((r<<11) | (g<<5) | (b));  
    
  }
}
