 /***************************************************************
 *                                                              *  
 *    o_gfxm.c                                                  *
 *    graphic modes and color ordering                          *
 *                                                              *
 *    (G) corTeX / Optimum   1999-01-30                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/


#include "o_gfxm.h"
#include <stdio.h>


static char * MODE_text[] = { "MODE_??? 0",
				    "MODE_RGBA_8888",    //1
				    "MODE_BGRA_8888",
				    "MODE_ARGB_8888",
				    "MODE_ABGR_8888",     //4
				    "MODE_??? 5",
				    "MODE_??? 6",
				    "MODE_??? 7",

				    "MODE_RGB_888",      // 8
				    "MODE_BGR_888",
				    "MODE_RGB_565",
				    "MODE_RGB_555",     //11
				    "MODE_??? 12",
				    "MODE_??? 13",
				    "MODE_??? 14",
				    "MODE_??? 15",
				    "MODE_PAL_8",      //16
				    "MODE_GRAY_8",
				    "MODE_RGB_332"};
  



int ofn_GetColorOrder( int rmask, int gmask, int bmask){
  int bitm = 0;

  // assume rgb are not intermixed ;) 
  bitm |= (rmask<gmask);
  bitm |= (rmask<bmask) << 1;
  bitm |= (gmask<bmask) << 2;

  switch (bitm) {

  case 2: return (MODE_XXX);  // note that these case should be impossible
  case 5: return (MODE_XXX);  // whatever rgbmask you provide !!! 

  default: return (bitm);  // by construct of chosen #defines
    
  }
  
}

int   ofn_GetGfxMode(int bpp, int depth,  int rmask, int gmask, int bmask){
  int order;
  int tmp_mode;

  tmp_mode = MODE_UNKNOWN;

  order = ofn_GetColorOrder(rmask, gmask, bmask);

  switch (depth){
  case 8:
    tmp_mode = MODE_PAL_8;
    break;

  case 15:
      switch (order) {
      case MODE_RGB : tmp_mode = MODE_RGB_555;
	              break;
      }
      break;
  case 16:
      
      switch (order) {
      case MODE_RGB : tmp_mode = MODE_RGB_565;
	              break;
      }
      break;
  case 24:
  case 32:  // depth==32 shouldnt exist... no ?
    switch (bpp){
    case 24:
      switch (order) {
      case MODE_RGB : tmp_mode = MODE_RGB_888 ;
                      break;
      case MODE_BGR : tmp_mode = MODE_BGR_888 ;
                      break;
      }
      break;
    case 32:
      switch (order) {
      case MODE_RGB : tmp_mode = ((bmask&1) ? MODE_ARGB_8888 :MODE_RGBA_8888 );
                      break;
      case MODE_BGR : tmp_mode = ((rmask&1) ? MODE_ABGR_8888 :MODE_BGRA_8888 );
                      break;
      }
      break;
    }
    break;
    
  }

  // ultimately, there should be modification to the return value
  // depending on the ByteOrder of the display....

  return (tmp_mode);

}



// return the component order of the specified MODE_ 
int ofn_GetOrderOfMode( int gfxMode){
  switch (gfxMode){
  case MODE_RGBA_8888:
  case MODE_ARGB_8888:
  case MODE_RGB_888:
  case MODE_RGB_565:
  case MODE_RGB_555:
  case MODE_RGB_332:
    return (MODE_RGB);

  case  MODE_BGRA_8888:
  case  MODE_ABGR_8888:
    return (MODE_BGR);

  default:
    return (MODE_XXX);
  }
}

// return the bpp (for memory allocation..) of the mode
int ofn_GetBppOfMode( int gfxMode){
  switch (gfxMode){
  case MODE_RGBA_8888:
  case MODE_ARGB_8888:
  case MODE_BGRA_8888:
  case MODE_ABGR_8888:
    return (32);

  case MODE_RGB_888:
    return (24);

  case MODE_RGB_565:
  case MODE_RGB_555:
    return (16);

  case MODE_PAL_8 :
  case MODE_GRAY_8 :
  case MODE_RGB_332 :
    return (8);

  default:
    return (8);          // What ? 
    // or 0 ?......

  }
}

// return the color depth
int ofn_GetDepthOfMode( int gfxMode){
 switch (gfxMode){
  case MODE_RGBA_8888:
  case MODE_ARGB_8888:
  case MODE_BGRA_8888:
  case MODE_ABGR_8888:
  case MODE_RGB_888:
    return (24);

  case MODE_RGB_555:
    return (16);

  case MODE_RGB_565:
    return (15);

  case MODE_PAL_8 :
  case MODE_GRAY_8 :
  case MODE_RGB_332 :
    return (8);

  default:
    return (8);          // What ? 
    // or 0 ?......

  }
}


char *ofn_GetNameOfMode(int gfxMode){
  if (gfxMode >=0 && gfxMode <=MODE_MAX)
    return (MODE_text[gfxMode]);
  else
    return ("Mode What ??");
}


/**********************/
/* Conversion Wrapper */

void ofn_ConvertMode(void *source, void *dest,
		   int sMode, int dMode,
		   int nbpixels,
		   int special,
		   ...){
  va_list ap;
  uint8 *p8;
  uint16 *p16;
  uint32 *p32;

  va_start(ap, special);


  if (dMode < 1 || dMode > MODE_MAX || 
      sMode < 1 || sMode > MODE_MAX ){
    fprintf(stderr,"Invalide mode (out of range) '%d->%d'\n",
	    sMode,dMode);
    return;
  }

  if (sMode == dMode) { // Yeah right !!!
    memcpy(dest, source, nbpixels * ofn_GetBppOfMode(sMode)/8);
    return;
  }

  
  switch (dMode){
  case MODE_PAL_8:
  case MODE_GRAY_8:
  case MODE_RGB_332:
    if (sMode == MODE_RGB_565){
      switch (special){
      case CONVERT_DEFAULT:
      case CONVERT_COMPUTE:
	ofn_cvrt_RGB_565_RGB_332((uint16 *)source, (uint8 *)dest, nbpixels);
	break;
      case CONVERT_PALETTE:
	p8 = va_arg(ap, uint8 *);
	ofn_cvrt_P_16_P_8((uint16 *)source, (uint8 *)dest, nbpixels, p8);
	break;
      default:
	fprintf(stderr,"Sorry, unsupported display format : %s\n",ofn_GetNameOfMode(dMode));
      }
      va_end(ap);
      return;
    }
    if (sMode == MODE_ARGB_8888){
      ofn_cvrt_ARGB_8888_RGB_332(source, dest, nbpixels);
      va_end(ap);
      return;
    }

    if (sMode == MODE_BGRA_8888){
      ofn_cvrt_BGRA_8888_RGB_332(source, dest, nbpixels);
      va_end(ap);
      return;
    }
    break;
  case MODE_RGB_555:
    if (sMode ==  MODE_ABGR_8888){
      ofn_cvrt_ABGR_8888_RGB_555(source, dest, nbpixels);
      va_end(ap);
      return;
    }
  if (sMode ==  MODE_RGB_565){
      ofn_cvrt_RGB_565_RGB_555(source, dest, nbpixels);
      va_end(ap);
      return;
    }
  }

  switch (sMode){   // XXXXXXXX putain c'est redondant, merde !!!!!
  case MODE_PAL_8:
  case MODE_GRAY_8:
  case MODE_RGB_332:
    switch (ofn_GetBppOfMode(dMode)){
    case 15:
    case 16:
      p16 = va_arg(ap, uint16 *);
      ofn_cvrt_P_8_P_16((uint8 *)source, (uint16 *)dest, nbpixels, p16);
      break;
    case 32:
      p32 = va_arg(ap, uint32 *);
      ofn_cvrt_P_8_P_32((uint8 *)source, (uint32 *)dest, nbpixels, p32);
      break;
    default:
      fprintf(stderr,"Ooops, not handled combination : %s -> %s \n",
	      ofn_GetNameOfMode(sMode), ofn_GetNameOfMode(dMode));
      break;

      
    }
    va_end(ap);
    return;
  };


  if (sMode == MODE_ARGB_8888 && dMode == MODE_RGB_565){
    ofn_cvrt_ARGB_8888_RGB_565(source, dest, nbpixels);
    va_end(ap);
    return;
  }

  if (sMode == MODE_ARGB_8888 && dMode == MODE_RGBA_8888){
    ofn_cvrt_ARGB_8888_RGBA_8888(source, dest, nbpixels);
    va_end(ap);
    return;
  }

  if (sMode == MODE_RGBA_8888 && dMode == MODE_ARGB_8888){
    ofn_cvrt_RGBA_8888_ARGB_8888(source, dest, nbpixels);
    va_end(ap);
    return;
  }

  if (sMode == MODE_ARGB_8888 && dMode == MODE_ABGR_8888){
    ofn_cvrt_ARGB_8888_ABGR_8888(source, dest, nbpixels);
    va_end(ap);
    return;
  }

  if (sMode == MODE_ABGR_8888 && dMode == MODE_ARGB_8888){
    ofn_cvrt_ABGR_8888_ARGB_8888(source, dest, nbpixels);
    va_end(ap);
    return;
  }




  if (sMode == MODE_ABGR_8888 && dMode == MODE_RGB_565){
    ofn_cvrt_ABGR_8888_RGB_565 (source, dest, nbpixels);
    va_end(ap);
    return;
  }
  if (sMode == MODE_RGBA_8888 && dMode == MODE_RGB_565){
    ofn_cvrt_RGBA_8888_RGB_565 (source, dest, nbpixels);
    va_end(ap);
    return;
  }



  if (sMode == MODE_RGB_565 && dMode == MODE_ARGB_8888){
    ofn_cvrt_RGB_565_ARGB_8888(source, dest, nbpixels);
    va_end(ap);
    return;
  }

 if (sMode == MODE_RGB_565 && dMode == MODE_ABGR_8888){
    ofn_cvrt_RGB_565_ABGR_8888(source, dest, nbpixels);
    va_end(ap);
    return;
  }


 fprintf(stderr,"Ooops, not handled combination : %s -> %s \n",
	 ofn_GetNameOfMode(sMode), ofn_GetNameOfMode(dMode));

 va_end(ap);
}




