 /***************************************************************
 *                                                              *  
 *    o_xutils.c                                                *
 *    basic Xwindow routines                                    *
 *                                                              *
 *    (G) corTeX / Optimum   1999-01-30                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/




#include "o_xutils.h"



static int o_o_XShmError;  // used in conjonction with shm_errhandler to test XShm

static int (*X_handler)(Display *, XErrorEvent *);

// used only to detect the availability of XShm
// If during the XShm-test of ofn_XOpen, XShm fails,
// X calls this function which will set a flag.
// Apparently there is no simple method to
// determine XShm capabilities.
// (these 10 lines of code taken from the SDL library
// (Simple Direct media Layer), http://www.devolution.com/~slouken/SDL/ )

static int shm_errhandler(Display *dis, XErrorEvent *ev){
  if ( ev->error_code == BadAccess ) {   // this is not very accurate...
    o_o_XShmError = 1;
    return(0);
  } else
    return(X_handler(dis,ev));
}





void ofn_XOpen (ot_display *Od, const char *Display, int testXShm ) {
  
  XPixmapFormatValues  *list;    // for list of pixmaps to choose from
  int                  count;
  
  
  Od->dis = XOpenDisplay  ( Display );
  
  if ( Od->dis == NULL)
    {
      fprintf ( stderr , "ofn_XOpen : Error :\n" );
      fprintf ( stderr , "  Cannot connect to display ( %s )\n",Display);
      exit (1);
    }

  Od-> screen = DefaultScreen (  Od->dis );
  Od-> depth  = DefaultDepth  (  Od->dis ,  Od->screen );
  Od-> width  = DisplayWidth  (  Od->dis ,  Od->screen );
  Od-> height = DisplayHeight (  Od->dis ,  Od->screen );

  Od->byteOrder = ImageByteOrder (Od->dis);

  Od->winRoot = DefaultRootWindow (  Od->dis );

  // try to find the good bpp from the pixmap list 
  // (any better way to do that ??)
  list = XListPixmapFormats( Od->dis, &count);
  if (list == NULL){
    fprintf ( stderr , " ofn_XOpen : Cannot get pixmap list\n");
    exit (1);
  }
  
  while (count && (list[count-1].depth != Od->depth))
      count--;

  if (!count){
    fprintf ( stderr , " ofn_XOpen : pixmap list doesn't contain screen depth\n");
    exit (1);
  }

  Od->bpp = list[count-1].bits_per_pixel;  // hard ... 

  XFree(list);

  if ( Od->depth > 8 ) {
    Od->rmask = DefaultVisual(Od->dis,Od->screen)->red_mask;
    Od->gmask = DefaultVisual(Od->dis,Od->screen)->green_mask;
    Od->bmask = DefaultVisual(Od->dis,Od->screen)->blue_mask;
  }
  else
    Od->rmask = Od->gmask = Od->bmask = 0;


  Od->gfxMode = ofn_GetGfxMode(Od->bpp, Od->depth,  Od->rmask, Od->gmask,  Od->bmask);




    

  Od->o_XShmAvailable = 0;      // default

  if (testXShm){


    XShmSegmentInfo shminfo;
    int error ;

    if (XShmQueryExtension(Od->dis) == 0) {
#ifdef OPL_DEBUG
      fprintf(stderr, "MIT shared memory extensions not present.\n");
#endif
      return ;
    }
    
    // We now need to actually test the XShm services
    // to see if it is available.
    // for this we install an X error handler, send
    // a XShm command, and look at the error flag.
    // If it was set by the error handler, then XShm is
    // not useable.
    // (I had the Docs for this, but the code is taken from
    // the SDL library because I am lazy)
    

    o_o_XShmError = 0;
    error = False;

    shminfo.shmid = shmget(IPC_PRIVATE, 32, // 32 : reserve some bytes, but its useless
			   IPC_CREAT|0777);
    if ( shminfo.shmid >= 0 ) {
      shminfo.shmaddr = (char *)shmat(shminfo.shmid, 0, 0);
      shminfo.readOnly = False;
      if ( shminfo.shmaddr != (char *)-1 ) {
	o_o_XShmError = 0;

	X_handler = XSetErrorHandler(shm_errhandler);
	XShmAttach(Od->dis, &shminfo);
	XSync(Od->dis, True);
	XSetErrorHandler(X_handler);

	if ( o_o_XShmError==0 )
	  XShmDetach(Od->dis, &shminfo);
	else
	  error = True;
	
	  shmdt(shminfo.shmaddr);
      } 
      else 
	error = True;
      
      shmctl(shminfo.shmid, IPC_RMID, NULL);
      
    }
    else 
      error = True;

      
    if (!error){  // OK, XShm works
      Od->o_XShmAvailable = 1;     
      Od->o_XShmCompletionType= XShmGetEventBase(Od->dis) + ShmCompletion;
    }
    else {
#ifdef OPL_DEBUG
      fprintf(stderr, "MIT shared memory extension present, but not used.\n(you are probably running this remotely).\n");
#endif
    }
      

  }  // test XShm stuff


}



void ofn_XClose (ot_display *Od){
  XCloseDisplay  ( Od->dis );     
}


/*******************/
/* Window routines */


void ofn_XOpenWindow(ot_display *Od, 
		   ot_top_window *Ow,             // returns into Ow
		   int Width, int Height,         // window size
		   char *Name                     // window titlebar
		   ){
  XGCValues            gcVal;
  unsigned long        gcMask;
  XColor black={0,0,0,0,0,0};


  Ow->od = Od;

  Ow->emptyPix       = XCreatePixmap(Od->dis,Od->winRoot,1,1,1);
  Ow->winAttr.cursor = XCreatePixmapCursor(Od->dis,Ow->emptyPix,Ow->emptyPix,&black,&black,0,0);

  Ow->winAttr.border_pixel     = BlackPixel ( Od->dis , Od->screen );
  Ow->winAttr.background_pixel = BlackPixel ( Od->dis , Od->screen );
  Ow->winMask                  = CWBackPixel | CWBorderPixel | CWCursor;

  Ow->win = XCreateWindow ( Od->dis , Od->winRoot , 
			    (Od->width/2)-(Width/2) , (Od->height/2)-(Height/2),
			    Width , Height , 
			    1 , Od->depth ,
			    InputOutput , CopyFromParent , 
			    Ow->winMask , &(Ow->winAttr) );

  XStoreName ( Od->dis , Ow->win , Name );

  XSelectInput ( Od->dis , Ow->win , KeyPressMask | KeyReleaseMask   // keyboard events
                                   | ExposureMask                    // parts of window becomes visiiible
		                   | StructureNotifyMask);           // to wait for the window to be mapped

  Ow->winHint.flags      = PPosition | PMinSize | PMaxSize ;
  Ow->winHint.x          = (Od->width/2)-(Width/2);
  Ow->winHint.y          = (Od->height/2)-(Height/2);
  Ow->winHint.max_width  = Ow->winHint.min_width  = Width;
  Ow->winHint.max_height = Ow->winHint.min_height = Height;
  XSetWMNormalHints ( Od->dis , Ow->win , &(Ow->winHint) );

  XClearWindow ( Od->dis , Ow->win );
  XMapRaised ( Od->dis , Ow->win );
  XFlush ( Od->dis );

  gcVal.foreground = 0;
  gcVal.background = 0;
  gcVal.function   = GXcopy;  // blitting : normal copy  ( perhaps you want to experiment with XOR ?)
  gcMask           = GCForeground | GCBackground | GCFunction;

  Ow->gc = XCreateGC ( Od->dis , Ow->win , gcMask , &gcVal );


  // wait for the window to be mapped on screen
  for(;;) {
    XEvent e;
    XNextEvent(Od->dis, &e);
    if (e.type == MapNotify)
      break;
  }


  if (Od->depth==8){
    int i;
    Ow->palettized = True;
    Ow->oxp = NEW(ot_xpalette);
    Ow->oxp->colormap=XCreateColormap(Od->dis,Ow->win,
				      DefaultVisual(Od->dis,Od->screen),AllocAll);
    for (i=0; i<256; i++){  // sets stuffs
      Ow->oxp->c[i].flags=DoRed|DoGreen|DoBlue;
      Ow->oxp->c[i].pixel=i;    // Hum ?...
    }

    //    ofn_XWindow332Cmap(Ow);   //  add useless flickering


    
    XSetWindowColormap(Od->dis,Ow->win,Ow->oxp->colormap);
    XInstallColormap(Od->dis,Ow->oxp->colormap);              
    // in this order - else cmap is for the whole screen

  }
  else {
    Ow->palettized = False;
    Ow->oxp = NULL;
  }



}




int  ofn_XKeyEvent (ot_top_window *Ow){
  XEvent xev;

  return (XCheckWindowEvent(Ow->od->dis, Ow->win, KeyPressMask ,&xev));
}


void ofn_XCloseWindow(ot_top_window *Ow){
  XFreeGC (Ow->od->dis, Ow->gc);
  XFreePixmap(Ow->od->dis, Ow->emptyPix);
  if (Ow->palettized && (Ow->oxp!=NULL)){
    XUninstallColormap(Ow->od->dis,Ow->oxp->colormap);
    XFreeColormap(Ow->od->dis,Ow->oxp->colormap);
    free(Ow->oxp); // yes !
  }
  XDestroyWindow ( Ow->od->dis , Ow->win );
}


void ofn_XWindowUpdateCmap(ot_top_window *Ow){

  if ((!Ow->palettized) || (Ow->oxp==NULL))
    return;
  
  XStoreColors(Ow->od->dis,Ow->oxp->colormap,Ow->oxp->c,256); 
  // The change is immediately visible (installed colormap)
}


// XXX this is probably useless
void ofn_XWindow332Cmap(ot_top_window *Ow){
  int i;

  if ((!Ow->palettized) || (Ow->oxp==NULL))
    return;

  for (i=0; i<256; i++){
    Ow->oxp->c[i].red  = (i & 0xe0) << 8;
    Ow->oxp->c[i].green= (i & 0x1c) << 11;
    Ow->oxp->c[i].blue = (i & 0x3 ) << 14;
  }
  ofn_XWindowUpdateCmap(Ow);
};




/******************/
/* Image routines */


void ofn_XCreateImage(ot_Ximage *Oxi,
		     ot_display *Od,
		     int Width, int Height){

  Oxi->od = Od;
  Oxi->flags = 0;
  Oxi->useShm = False;

#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XCreateImage\n");
#endif

  
  if (Oxi->od->o_XShmAvailable){
    Oxi->xim = XShmCreateImage(Od->dis,CopyFromParent , Od->depth, 
			      ZPixmap, 0,
			      &(Oxi->os.SHMInfo), Width, Height);
    
    Oxi->os.SHMInfo.shmid = shmget(IPC_PRIVATE, 
				   Oxi->xim->bytes_per_line*Oxi->xim->height, 
				   IPC_CREAT|0777);
   
    Oxi->xim->data = Oxi->data = Oxi->os.SHMInfo.shmaddr = (char *)shmat(Oxi->os.SHMInfo.shmid, 0, 0);
    Oxi->os.SHMInfo.readOnly = False;
    XShmAttach(Od->dis, &(Oxi->os.SHMInfo));
    Oxi->useShm = True;
#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XCreateImage : Xshm succesful\n");
#endif

  }
  else{
    Oxi->data= ( void * ) calloc ( Width*Height*Od->bpp/8,
				  1); 
    // I DO assume continuity between scanlines (no padding) because
    // main prog routines will do the same...
    // Keep your image width multiple of 4 or 8 and there is no
    // risk !

    Oxi->xim = XCreateImage ( Oxi->od->dis , CopyFromParent , Oxi->od->depth , 
			     ZPixmap , 0 ,
			     Oxi->data , Width , Height , 32 , 0 /* no skip */ );
    /* bitmap pad = 32 ... hum */
#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XCreateImage : standard ximage succesful\n");
#endif

  }

}

void ofn_XDestroyImage(ot_Ximage *Oxi){

#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XDestroyImage\n");
#endif


  if (Oxi->useShm){
    if (Oxi->flags & 1)
      ofn_XWaitShm(Oxi);  // read pending completion event
    XShmDetach(Oxi->od->dis, &(Oxi->os.SHMInfo));
    if (Oxi->os.SHMInfo.shmaddr)
      shmdt(Oxi->os.SHMInfo.shmaddr);
    if (Oxi->os.SHMInfo.shmid >= 0)
      shmctl(Oxi->os.SHMInfo.shmid, IPC_RMID, 0);   
#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XDestroyImage xshm ok\n");
#endif

  }
  XDestroyImage  ( Oxi->xim );

#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XDestroyImage std ok\n");
#endif

  
}

/************************/
/* send image to window */

void ofn_XPutImage(ot_Ximage *Oxi, ot_top_window *Ow, 
		 int X, int Y,            // start into Image 
		 int WX,int WY,           // destination into window 
		 int Width, int Height    // (0,0) to put all image 
		 ){

  if (Width==0)     Width = Oxi->xim->width;
  if (Height==0)    Height = Oxi->xim->height;

#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XPutImage\n");
#endif


  if (Oxi->useShm){
    if (Oxi->flags & 1)  ofn_XWaitShm(Oxi);

    XShmPutImage( Ow->od->dis, Ow->win, Ow->gc,
		  Oxi->xim, 
		  X, Y,
		  WX, WY, 
		  Width, Height, True );   /* and generate event on completion */
    Oxi->flags |= 1; // wait for completion
#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XPutImage Xshm ok\n");
#endif

  }
  else{
      XPutImage( Ow->od->dis, Ow->win, Ow->gc,
		 Oxi->xim, 
		 X, Y,
		 WX, WY, 
		 Width, Height ); 
#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XPutImage std ok\n");
#endif

  }

  XFlush( Ow->od->dis ); // yep
}


// read coordinates from the subwindow
void ofn_XPutSimpleImage(ot_Ximage *Oxi, ot_sub_window *Osw){
  ofn_XPutImage(Oxi,Osw->otop, 
		0,0,
		Osw->x, Osw->y,
		Osw->width, Osw->height);
	      
}



// private structure for ofn_WaitShm
typedef struct{
  int    o_XShmCompletionType;
  ShmSeg shmseg;
} o_o_EventShmWait;

// function to be passed to XIfEvent, by ofn_WaitShm.
// it returns true for a ShmCompletion event.
/*
** XXX : add a standard 'predicate' function with a ot_PredicateArg type,
** to be used in all the lib ?...
*/
static Bool o_o_WaitForShmCompletion(Display *d, XEvent *xev, char *arg) {
  if (xev->type != ((o_o_EventShmWait *)arg)->o_XShmCompletionType)
        return (False);
  if (((XShmCompletionEvent *)xev)->shmseg != ((o_o_EventShmWait *)arg)->shmseg)
        return (False);
    
  return (True);
}  

void ofn_XWaitShm (ot_Ximage *Oxi){
  XEvent xev;
  o_o_EventShmWait arg;

#ifdef DEBUG_XUTILS
  fprintf(stderr, "ofn_XWaitShm\n");
#endif

  //  return;

  Oxi->flags &= ~1; // no more completion waiting
  if (Oxi->useShm){

    arg.o_XShmCompletionType = Oxi->od->o_XShmCompletionType;
    arg.shmseg               = Oxi->os.SHMInfo.shmseg;

    XIfEvent( Oxi->od->dis,&xev, o_o_WaitForShmCompletion, (char*)(&arg));

  }


}



