/*
**
**  Open a window, create a 8bpp surface, sets its palette, blit it, close
**
*/ 



#include <stdio.h>
#include <math.h>

#include "o_std.h"
#include "o_types.h"
#include "o_xutils.h"
#include "o_surface.h"
#include "o_gfxm.h"



#define WIDTH  320
#define HEIGHT 240

void main(){
  ot_display    *ODis;
  ot_top_window *OWin;
  ot_surface    *OSurf;
  uint8         *tmpbuffer;
  ot_color      *palette;
  int x,y,i;

  // allocate these structures. 

  ODis = NEW(ot_display);
  OWin = NEW(ot_top_window);
  OSurf= NEW(ot_surface);
  palette = NEW_ARRAY(ot_color,256);

  // Open display and create a window

  ofn_XOpen(ODis, "",True);
  ofn_XOpenWindow(ODis, OWin, WIDTH, HEIGHT, "Example2 : palette");

  // see what Optilib has detected
  printf(" XShm extension : %s\n", 
	 ODis->o_XShmAvailable ? "In use": "Not in use");
  printf(" Display mode : %s\n",
	 ofn_GetNameOfMode(ODis->gfxMode));
  

  // set the palette
  for (i=0; i<256; i++){
    palette[i].r = (int)(i*0.7);
    palette[i].g = (int)(i*0.8);
    palette[i].b = (i*4)&255;
  } 

  // Create a 8bpp surface and attach it to the X window
   ofn_SurfaceCreate( OSurf, WIDTH, HEIGHT,  MODE_PAL_8);
   ofn_SurfaceAttachX( OSurf, OWin, 0,0); /* coordinates are (0,0) */

   
   // Send the palette *After* attaching the surface to X.
   ofn_SurfaceSetPalette(OSurf, palette);

   // you can now free the palette
   free(palette);
     


   // draw concentric circles

   tmpbuffer = (uint8 *)OSurf->buffer;
   for (y = 0; y< HEIGHT; y++)
     for (x = 0; x< WIDTH; x++)
       {
	 double radius;
	 radius = sqrt (  (y-HEIGHT/2)*(y-HEIGHT/2)
			 +(x-WIDTH/2)*(x-WIDTH/2));

	 *(tmpbuffer + x + WIDTH*y) = floor(radius);

       }

   // now we can display it
   ofn_SurfaceBlit(OSurf);
   sleep(5);    // so we have time to see something...


   // Release structures

   ofn_SurfaceDetach(OSurf);    // free stuffs allocated by attach()
   ofn_SurfaceDestroy(OSurf);   // clear remaining stuff
   free(OSurf);                 // *This is not done by Destroy*


   ofn_XCloseWindow(OWin);
   ofn_XClose(ODis);

   free(OWin);
   free(ODis);
     

}

