/*
**
**  create a surface list of two different color mode;
**  then display the list several time changing the flags
**
*/ 



#include <stdio.h>
#include <math.h>

#include "o_std.h"
#include "o_types.h"
#include "o_xutils.h"
#include "o_surface.h"
#include "o_gfxm.h"



#define WIDTH  320
#define HEIGHT 240


void Draw8(ot_surface *OS8){
  uint8 *tmpbuffer;
  int x,y;

  tmpbuffer = (uint8 *)OS8->buffer;
   for (y = 0; y< HEIGHT; y++)
     for (x = 0; x< WIDTH/2; x++)
       {
	 double radius;
	 radius = sqrt (  (y-HEIGHT/2)*(y-HEIGHT/2)
			 +(x-WIDTH/4)*(x-WIDTH/4));

	 *(tmpbuffer + x + y*(WIDTH/2)) += floor(radius);

       }
}


void Draw32(ot_surface *OS32){
  uint32 *tmpbuffer;
  int x,y;

  tmpbuffer = (uint32 *)OS32->buffer;
   for (y = 0; y< HEIGHT; y++)
     for (x = 0; x< WIDTH/2; x++)
       {
	 *(tmpbuffer + x + y*(WIDTH/2)) +=   ((x^y) & 0xff)
                                          | (((x&y) & 0xff)<<8);

       }
}


void main(){
  ot_display      *ODis;
  ot_top_window   *OWin;
  ot_surface      *OSurf[2];
  int              index[2];  // handle of the surfaces in the list;
  ot_surface_list *OSList;
  ot_color        *palette;
  int              i;
  // allocate these structures. 

  ODis = NEW(ot_display);
  OWin = NEW(ot_top_window);
  OSurf[0]= NEW(ot_surface);
  OSurf[1]= NEW(ot_surface);
  OSList  = NEW(ot_surface_list);
  palette = NEW_ARRAY(ot_color,256);

  // Open display and create a window

  ofn_XOpen(ODis, "",True);
  ofn_XOpenWindow(ODis, OWin, WIDTH, HEIGHT, "Example3 : list");

  // see what Optilib has detected
  printf(" XShm extension : %s\n", 
	 ODis->o_XShmAvailable ? "In use": "Not in use");
  printf(" Display mode : %s\n",
	 ofn_GetNameOfMode(ODis->gfxMode));

  // different palettes cannot be displayed simultaneously on
  // 8bpp servers... 
  if (ODis->gfxMode == MODE_PAL_8)
    printf(" Warning, your 8bpp server will only display trash.... sorry...\n");
  

  // set the palette for the first surface
  for (i=0; i<256; i++){
    palette[i].r = (int)(i*0.7);
    palette[i].g = (int)(i*0.8);
    palette[i].b = (i*4)&255;
  } 

  // Create a 8bpp surface and attach it to the X window
   ofn_SurfaceCreate( OSurf[0], WIDTH/2, HEIGHT,  MODE_PAL_8);
   ofn_SurfaceAttachX( OSurf[0], OWin, 0,0); /* coordinates are (0,0) */
   ofn_SurfaceSetPalette(OSurf[0], palette);

   // you can now free the palette
   free(palette);


  // Create a truecolor image
   ofn_SurfaceCreate( OSurf[1], WIDTH/2, HEIGHT,  MODE_ARGB_8888);
   ofn_SurfaceAttachX( OSurf[1], OWin, WIDTH/2,0); 


   index[0] = ofn_SurfaceListAddElement(OSList, OSurf[0], OPL_SLIST_VISIBLE | OPL_SLIST_UPDATED);
   index[1] = ofn_SurfaceListAddElement(OSList, OSurf[1], OPL_SLIST_VISIBLE | OPL_SLIST_UPDATED);




   Draw8(OSurf[0]);
   Draw32(OSurf[1]);


 
   

   // now we can display it
   ofn_SurfaceListBlit(OSList, OPL_SLIST_VISIBLE | OPL_SLIST_UPDATED);
   sleep(1);
 

   // modify both surfaces and display them
   Draw8(OSurf[0]);
   Draw32(OSurf[1]);
   ofn_SurfaceListBlit(OSList, OPL_SLIST_VISIBLE | OPL_SLIST_UPDATED);
   sleep(1);


   // modify both surfaces but mark the right one as 'not updated' :
   // it won't be displayed
   Draw8(OSurf[0]);
   Draw32(OSurf[1]);
   OSList->flags[index[1]] &= ~OPL_SLIST_UPDATED;
   ofn_SurfaceListBlit(OSList, OPL_SLIST_VISIBLE | OPL_SLIST_UPDATED);
   sleep(1);

   // Change the mask passed to SurfaceListBlit : display everything
   // visible, updated or not : the right image is then displayed
   ofn_SurfaceListBlit(OSList, OPL_SLIST_VISIBLE);
   sleep(1);

   
   sleep(1);



   // Release structures
   ofn_SurfaceListDestroyAll(OSList); 
   free(OSList); 
   /* ofn_SurfaceListDestroyAll detach, destroy and free the
    * associated surfaces.
    */


   ofn_XCloseWindow(OWin);
   ofn_XClose(ODis);

   free(OWin);
   free(ODis);
     

}







